// CS205 Fall 2006
// SimObject.java

import java.util.Enumeration;
import java.util.Vector;
import java.awt.Color;

public class SimObject {
	// OVERVIEW: A SimObject is an object than represents a simulator object.
	// It has a grid (a circular reference to the grid containing this object),
	// location (integer x and y), initialization state (boolean
	// that is true if it has beein initialized).
	//
	// A typical SimObject is < grid, x, y, initialized >.
	//

	protected Grid grid;

	// subclasses have access to mx and my
	protected int mx;
	protected int my;

	// AF(c) = < grid, mx, my, isInitialized >
	// RI(c) =
	// if isIntialized then
	// grid != null
	// && grid.validLocation (mx, my)
	// && grid.getObjectAt (mx, my) == this

	// If isInitialized is true, the grid is not null.

	public SimObject()
	// EFFECTS: Creates a new, uninitalized SimObject.
	{
	}

	final public void init(int x, int y, /* nonnull */Grid grid)
	// REQUIRES: init has not previously been called for this.
	//    The cell is at (x, y) on grid.
	// MODIFIES: this
	// EFFECTS: Initializes the cell at x, y on grid.
	// NOTE: This method is final, that means subtypes cannot override this
	//    method.
	{
		this.mx = x;
		this.my = y;
		this.grid = grid;
	}

	public /*nonnull*/Color getColor()
	// EFFECTS: Returns the yor that represents this SimObject, for
	// painting the grid.
	{
		return Color.green;
	}

	final public int getX()
	// REQUIRES: init has previously been called for this.
	// EFFECTS: Returns the x number that this cell is located in.
	{
		return mx;
	}

	final public Coordinate getLocation()
	// REQUIRES: this is initialized.
	// EFFECTS: Returns the y location that this cell is located in.
	{
		return new Coordinate(mx, my);
	}

	final public int getY()
	// REQUIRES: this is initialized.
	// EFFECTS: Returns the y location that this cell is located in.
	{
		return my;
	}

	synchronized public /*nonnull*/ Enumeration<SimObject> getNeighbors() {
		// REQUIRES: The grid must be locked as long as the result of getNeighbors
		//    is used. Otherwise, the neighbors could change if another
		//    object moves!
		// EFFECTS: Returns an Enumeration that yields all the objects adjacent to
		//    this in the grid.
		Vector<SimObject> objects = new Vector<SimObject>();

		for (int x = mx - 1; x <= mx + 1; x++) {
			for (int y = my - 1; y <= my + 1; y++) {
				if (x != mx || y != my) { // don't include this
					if (grid.validLocation(x, y)) {
						SimObject obj = grid.getObjectAt(x, y);
						if (obj != null) {
							objects.addElement(obj);
						}
					}
				}
			}
		}

		return objects.elements();
	} 
}